/*
 * 08/28/2003
 *
 * bmp.h
 * Structures de donnes et constantes pour les images
 * aux formats BMP
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _BMP_H
#define _BMP_H

#ifdef WIN32
#include <windows.h>
#include <wingdi.h>
#endif // WIN32

// Pour que ces dclarations fonctionnent en C++ et en C
#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

#ifndef WIN32
typedef struct                       // BMP file header structure
{
	unsigned short bfType;           // Magic number for file
	unsigned int   bfSize;           // Size of file
	unsigned short bfReserved1;      // Reserved
	unsigned short bfReserved2;      // ...
	unsigned int   bfOffBits;        // Offset to bitmap data
} BITMAPFILEHEADER;

#define BF_TYPE 0x4D42             // "MB"

typedef struct                       // BMP file info structure
{
	unsigned int   biSize;           // Size of info header
	int            biWidth;          // Width of image
	int            biHeight;         // Height of image
	unsigned short biPlanes;         // Number of color planes
	unsigned short biBitCount;       // Number of bits per pixel
	unsigned int   biCompression;    // Type of compression to use
	unsigned int   biSizeImage;      // Size of image data
	int            biXPelsPerMeter;  // X pixels per meter
	int            biYPelsPerMeter;  // Y pixels per meter
	unsigned int   biClrUsed;        // Number of colors used
	unsigned int   biClrImportant;   // Number of important colors
} BITMAPINFOHEADER;


// Constants for the biCompression field...
#  define BI_RGB       0             // No compression - straight BGR data
#  define BI_RLE8      1             // 8-bit run-length compression
#  define BI_RLE4      2             // 4-bit run-length compression
#  define BI_BITFIELDS 3             // RGB bitmap with RGB masks

typedef struct                       // Colormap entry structure
{
	unsigned char  rgbBlue;          // Blue value
	unsigned char  rgbGreen;         // Green value
	unsigned char  rgbRed;           // Red value
	unsigned char  rgbReserved;      // Reserved
} RGBQUAD;

typedef struct                       // Bitmap information structure
{
	BITMAPINFOHEADER bmiHeader;      // Image header
	RGBQUAD          bmiColors[256]; // Image colormap
} BITMAPINFO;
#  endif // !WIN32

#ifdef __cplusplus
}
#endif // __cplusplus

#endif
