/*
 * 08/06/2003
 *
 * histoboxFL.cpp
 * Box (FLTK) permettant d'afficher les histogrammes d'une image
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "histoboxFL.h"

// ============
// Constructeur
// ============
CHistoBoxFL::CHistoBoxFL(int X,int Y,int W,int H,const char* L) : Fl_Widget(X,Y,W,H,L) 
{
	m_height = 50 + 4*HISTOS_SPACE;
	m_width = 10;
	m_imgPath = NULL;
	m_imgInfos = new char[64];
	m_histoL = new char[64];
	m_histoR = new char[64];
	m_histoG = new char[64];
	m_histoB = new char[64];
	m_histo = new CHistogramme();
	m_aff[0] = 1;
	m_aff[1] = 1;
	m_aff[2] = 1;
	m_aff[3] = 1;	
}

// ===========
// Destructeur
// ===========
CHistoBoxFL::~CHistoBoxFL()
{
	delete m_imgInfos;
	delete m_histoL;
	delete m_histoR;
	delete m_histoG;
	delete m_histoB;
	delete m_histo;
}

// ====================================
// Calcule les histogrammes d'une image
// ====================================
void CHistoBoxFL::SetImage(CImage *img) 
{ 
	m_histo->CalcHistos(img);
	sprintf(m_imgInfos, "Width = %d, Height = %d, Bits per pixel = %d", img->width, img->height, img->bitCount);
	sprintf(m_histoL, "Couche luminosit : Moyenne = %d, Min = %d, Max = %d", m_histo->moy_l, m_histo->min_l, m_histo->max_l);
	sprintf(m_histoR, "Couche rouge : Moyenne = %d, Min = %d, Max = %d", m_histo->moy_r, m_histo->min_r, m_histo->max_r);
	sprintf(m_histoG, "Couche verte : Moyenne = %d, Min = %d, Max = %d", m_histo->moy_g, m_histo->min_g, m_histo->max_g);
	sprintf(m_histoB, "Couche bleue : Moyenne = %d, Min = %d, Max = %d", m_histo->moy_b, m_histo->min_b, m_histo->max_b);
	redraw();
}

// ===================================
// Change le chemin d'accs de l'image
// ===================================
void  CHistoBoxFL::SetImagePath(char *path) 
{ 
	m_imgPath = path; 
	fl_font(FL_COURIER, 14); 
	int sw = (int)fl_width(m_imgPath)+10;
	m_width =  sw > 530 ? sw : 530; 
}

// ==============================================
// Permet d'afficher ou non certains histogrammes
// 0=Luminosie, 1=rouge, 2=vert, 3=bleu
// ==============================================
void CHistoBoxFL::SetAff(int i, int value) 
{ 
	m_aff[i] = value; 
	if (value) m_height += HISTOS_SPACE; 
	else m_height -= HISTOS_SPACE; 
}

// ========================
// Dessine les histogrammes
// ========================
void CHistoBoxFL::draw() {
	
	if (m_imgPath != NULL) { 
		
		size(m_width,m_height);
		fl_color(FL_BLACK);
		fl_font(FL_COURIER, 14);
		fl_draw(m_imgPath, x()+10, y()+20);
		fl_draw(m_imgInfos, x()+10, y()+40); 

		DrawHistos(x(), y()+HISTOS_SPACE+5);
	}
}

// ==================================
// Ferme l'affichage des histogrammes
// ==================================
void CHistoBoxFL::Close()
{
	m_imgPath = NULL;
	size(0,0);
}

// ========================
// Dessine les histogrammes
// ========================
void CHistoBoxFL::DrawHistos(int x, int y)
{
	int l,r,g,b;
	int p = 0;
	m_histo->GetMaxHauteurs(&l, &r, &g, &b);
	fl_push_matrix();
	fl_translate(x, y);
	if (m_aff[0] || m_aff[1] || m_aff[2] || m_aff[3])
	for (int i=0; i<256; i++) 
	{	
		p = 0;
		if (m_aff[0]) {
			if (m_histo->histo_l[i] != 0) {
				fl_color(FL_BLACK);
				fl_begin_line();
				fl_vertex(10+i*2,HISTOS_SPACE*p);
				fl_vertex(10+i*2,HISTOS_SPACE*p -((float)(float)m_histo->histo_l[i]/(float)l)*HISTOS_HEIGHT);
				fl_end_line();
			}
			fl_color(fl_rgb_color(i));
			fl_begin_polygon();
			fl_vertex(10+i*2,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2,HISTOS_SPACE*p+10);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+10);
			fl_end_polygon();
			p++;
		}

		if (m_aff[1]) {
			if (m_histo->histo_r[i] != 0) {
				fl_color(FL_BLACK);
				fl_begin_line();
				fl_vertex(10+i*2,HISTOS_SPACE*p);
				fl_vertex(10+i*2,HISTOS_SPACE*p -((float)(float)m_histo->histo_r[i]/(float)r)*HISTOS_HEIGHT);
				fl_end_line();
			}
			fl_color(i,0,0);
			fl_begin_polygon();
			fl_vertex(10+i*2,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2,HISTOS_SPACE*p+10);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+10);
			fl_end_polygon();
			p++;
		}

		if (m_aff[2]) {
			if (m_histo->histo_g[i] != 0) {
				fl_color(FL_BLACK);
				fl_begin_line();
				fl_vertex(10+i*2,HISTOS_SPACE*p);
				fl_vertex(10+i*2,HISTOS_SPACE*p -((float)(float)m_histo->histo_g[i]/(float)g)*HISTOS_HEIGHT);
				fl_end_line();
			}
			fl_color(0,i,0);
			fl_begin_polygon();
			fl_vertex(10+i*2,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2,HISTOS_SPACE*p+10);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+10);
			fl_end_polygon();
			p++;
		}

		if (m_aff[3]) {
			if (m_histo->histo_b[i] != 0) {
				fl_color(FL_BLACK);
				fl_begin_line();
				fl_vertex(10+i*2,HISTOS_SPACE*p);
				fl_vertex(10+i*2,HISTOS_SPACE*p -((float)(float)m_histo->histo_b[i]/(float)b)*HISTOS_HEIGHT);
				fl_end_line();
			}
			fl_color(0,0,i);
			fl_begin_polygon();
			fl_vertex(10+i*2,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2,HISTOS_SPACE*p+10);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+2);
			fl_vertex(10+i*2+1,HISTOS_SPACE*p+10);
			fl_end_polygon();
			p++;
		}
	}
	fl_pop_matrix();

	fl_color(FL_BLACK);
	p = 0;
	if (m_aff[0]) { fl_draw(this->m_histoL, x+10, HISTOS_SPACE*p+y+25); p++; }
	if (m_aff[1]) { fl_draw(this->m_histoR, x+10, HISTOS_SPACE*p+y+25); p++; }
	if (m_aff[2]) { fl_draw(this->m_histoG, x+10, HISTOS_SPACE*p+y+25); p++; }
	if (m_aff[3]) { fl_draw(this->m_histoB, x+10, HISTOS_SPACE*p+y+25); p++; }
}

