/*
 * 08/06/2003
 *
 * histoboxFL.h
 * Box (FLTK) permettant d'afficher les histogrammes d'une image
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _HISTOBOXFL_H
#define _HISTOBOXFL_H

#include <FL/Fl.H>
#include <FL/fl_draw.h>
#include <FL/Fl_Widget.H>

#include "image.h"
#include "histogramme.h"

// Hauteur des histogrammes
#define HISTOS_HEIGHT 150.0f
#define HISTOS_SPACE (int)(HISTOS_HEIGHT+50)

// ##################
// Classe CHistoBoxFL
// ##################
class CHistoBoxFL : public Fl_Widget
{
  
public:

	// ============
	// Constructeur
	// ============
	CHistoBoxFL(int X,int Y,int W,int H,const char* L=0);

	// ===========
	// Destructeur
	// ===========
	~CHistoBoxFL();

	// ========================
	// Dessine les histogrammes
	// ========================
	void draw();
 
	// ====================================
	// Calcule les histogrammes d'une image
	// ====================================
	void SetImage(CImage *img);

	// ===================================
	// Change le chemin d'accs de l'image
	// ===================================
	void SetImagePath(char *path);

	// =====================================
	// Retourne le chemin d'accs de l'image
	// =====================================
	char * GetImagePath() { return m_imgPath; }

	// ==============================================
	// Permet d'afficher ou non certains histogrammes
	// 0=Luminosie, 1=rouge, 2=vert, 3=bleu
	// ==============================================
	void SetAff(int i, int value);

	// ==================================
	// Ferme l'affichage des histogrammes
	// ==================================
	void Close();
	
private:

	// ========================
	// Dessine les histogrammes
	// ========================
	void DrawHistos(int x, int y);

	// Chemin d'acces de l'image
	char *m_imgPath;
	// Informations sur l'image w*h d
	char *m_imgInfos;
	// Informations sur l'histogramme de la couche luminosite
	char *m_histoL;
	// Informations sur l'histogramme de la couche rouge
	char *m_histoR;
	// Informations sur l'histogramme de la couche verte
	char *m_histoG;
	// Informations sur l'histogramme de la couche bleue
	char *m_histoB;
	// Hauteur et largeur de la zone d'affichage
	int m_height;
	int m_width;
	// Histogrammes  affiher
	int m_aff[4];
	// Histogrammes
	CHistogramme *m_histo;
};

#endif




