/*
 * 08/06/2003
 *
 * histogramme.cpp
 * Classe permettant de calculer les histogrammes d'une image
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "histogramme.h"

// ============
// Constructeur
// ============
CHistogramme::CHistogramme()
{
	histo_l = new int[256];
	histo_r = new int[256];
	histo_g = new int[256];
	histo_b = new int[256];
}

// ===========
// Destructeur
// ===========
CHistogramme::~CHistogramme()
{
	delete histo_l;
	delete histo_r;
	delete histo_g;
	delete histo_b;
}

// ===================================
// Calcule les histogrammes de l'image
// ===================================
void CHistogramme::CalcHistos(CImage *img)
{
	int ng;
	long i;
	int np = (img->bitCount/8);
	long d = img->width * img->height * np;

	moy_l = -1;
	moy_r = -1;
	moy_g = -1;
	moy_b = -1;
	max_l = -1;
	max_r = -1;
	max_g = -1;
	max_b = -1;
	min_l = 999;
	min_r = 999;
	min_g = 999;
	min_b = 999;

	for (i=0 ; i<256 ; i++) {
		histo_l[i] = 0;
		histo_r[i] = 0;
		histo_g[i] = 0;
		histo_b[i] = 0;
	}

	for (i=0; i<d; i+=np) {
		if (np == 3) ng = (int)(img->data[i]*0.707+img->data[i+1]*0.202+img->data[i+2]*0.091);
		else ng = img->data[i];
		histo_l[ng]++;
		moy_l += ng;
		if (max_l < ng) max_l = ng;
		if (min_l > ng) min_l = ng;
		if (img->bitCount == 24) {
			histo_r[img->data[i]]++;
			histo_g[img->data[i+1]]++;
			histo_b[img->data[i+2]]++;
			moy_r += img->data[i];
			moy_g += img->data[i+1];
			moy_b += img->data[i+2];
			if (max_r < img->data[i]) max_r = img->data[i];
			if (min_r > img->data[i]) min_r = img->data[i];
			if (max_g < img->data[i+1]) max_g = img->data[i+1];
			if (min_g > img->data[i+1]) min_g = img->data[i+1];
			if (max_b < img->data[i+2]) max_b = img->data[i+2];
			if (min_b > img->data[i+2]) min_b = img->data[i+2];
		}
	}

	d = d / np;
	moy_l = (int)((float)moy_l / (float)d);
	moy_r = (int)((float)moy_r / (float)d);
	moy_g = (int)((float)moy_g / (float)d);
	moy_b = (int)((float)moy_b / (float)d);

}

// ===============================================
// Retourne les hauteurs maximums des histogrammes
// en nombre de pixels
// ===============================================
void CHistogramme::GetMaxHauteurs(int *l, int *r, int *g, int *b)
{
	*l = -1; *r = -1; *g = -1; *b = -1;

	for (int i=0 ; i<256 ; i++) {
		if (*l < histo_l[i]) *l = histo_l[i];
		if (*r < histo_r[i]) *r = histo_r[i];
		if (*g < histo_g[i]) *g = histo_g[i];
		if (*b < histo_b[i]) *b = histo_b[i];
	}
}

