/*
 * 08/06/2003
 *
 * image.h
 * Classe permettant de stocker une image
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _IMAGE_H
#define _IMAGE_H

#ifdef _WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include <GL/glu.h>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "bmp.h"

#ifndef GL_CLAMP_TO_EDGE
#define GL_CLAMP_TO_EDGE 0x812F	
#endif

// Codes erreurs de retour des fonctions
enum IMGErrors
{
	IMG_FILE_NOT_FOUND,
	IMG_BAD_IMAGE_TYPE,
	IMG_BAD_BITS,
	IMG_BAD_DATA,
	IMG_FILE_NOT_CREATE,
	IMG_BAD_SIZE,
	IMG_OK
};

// Diffrents types d'images
enum IMGTypes
{
	BMP,
	PPM,
	PGM,
	TGA,
	INVALID
};

// #############
// Classe CImage
// #############
class CImage
{
public:

	// ============
	// Constructeur
	// ============
	CImage();

	// ============
	// Constructeur
	// ============
	CImage(int w, int h, int d);

	// ============
	// Constructeur
	// ============
	CImage(unsigned char *da, int w, int h, int d);

	// ===========
	// Destructeur
	// ===========
	~CImage();

	// =============================
	// Retourne une copie de l'image
	// =============================
	CImage * Copy();

	// ================================================================
	// Fonction permettant de tester si un entier est une puisance de 2
	// ================================================================
	bool static CheckSize(int x);

	// ====================================================================
	// Retourne le format OpenGL de l'image (GL_RGB, GL_RGBA, GL_LUMINANCE)
	// ====================================================================
	GLenum GetTextureFormat();

	// ==================================================
	// Retourne le type de l'image en fonction de son nom
	// ==================================================
	int static GetType(const char *name);

	// ================================================
	// Fonction permettant de lire un fichier image TGA
	// 8, 16, 24 ou 32 bits
	// Le format 32 bits est convertit en 24 bits
	// ================================================
	int static LoadTGA(CImage *img, const char *filename);

	// ============================================================
	// Fonction enregistrant une image TGA dans le fichier filename
	// ============================================================
	int static SaveTGA(CImage *img, const char *filename);

	// ============================================================
	// Fonction enregistrant une image BMP dans le fichier filename
	// ============================================================
	int static SaveBMP(CImage *img, const char *filename);

	// Nombre de pixels en largeur de l'image
	short int width;

	// Nombre de pixels en hauteur de l'image
	short int height;

	// Nombre de bits par pixel (RGB = 24, RGBA = 32, Niveaux de gris = 8)
	unsigned char bitCount;

	// Tableau contenant les pixels de l'image
	unsigned char *data;

};

#endif

