/*
 * 08/06/2003
 *
 * imageboxFL.cpp
 * Box (FLTK) permettant d'afficher une image
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "imageboxFL.h"

// ============
// Constructeur
// ============
CImageBoxFL::CImageBoxFL(int X,int Y,int W,int H,const char* L) : Fl_Box(X,Y,W,H,L) 
{
	size(0, 0);
	m_img = NULL;
	m_imgAff = NULL;
	m_imgUndo = NULL;
	image(NULL);
	m_zoom = 1;
	m_curseur[0] = 0;
	m_curseur[1] = 0;
}

// ===========
// Destructeur
// ===========
CImageBoxFL::~CImageBoxFL()
{
	if (m_img) delete m_img;
	if (m_imgAff) delete m_imgAff;
	if (m_imgUndo) delete m_imgUndo;
	size(0, 0);
}

// ===============
// Dessine l'image
// ===============
void CImageBoxFL::draw()
{
	if (m_imgAff) {
		fl_draw_image(m_imgAff->data, x(), y(), m_imgAff->width, m_imgAff->height, m_img->bitCount/8); 
		fl_color(FL_BLACK);	
		fl_rect(m_curseur[0]+x()-4,m_curseur[1]+y()-4, 9, 9);
		fl_color(FL_RED);	
		fl_line(m_curseur[0]+x()-3,m_curseur[1]+y()-3,m_curseur[0]+x()+3,m_curseur[1]+y()+3);
		fl_line(m_curseur[0]+x()-3,m_curseur[1]+y()+3,m_curseur[0]+x()+3,m_curseur[1]+y()-3);
	}
}

// =========================
// Change l'image a afficher
// =========================
bool CImageBoxFL::SetImage(const char *name)
{
	if (m_img) delete m_img;
	if (m_imgAff) delete m_imgAff;
	if (m_imgUndo) delete m_imgUndo;
	m_img = NULL;
	m_imgAff = NULL;
	m_imgUndo = NULL;
	int error = IMG_OK;
	
	// == Chargement de l'image ==
	int type = CImage::GetType(name);
	switch (type) 
	{
		case TGA:
			m_img = new CImage();
			error = CImage::LoadTGA(m_img, name);
			//if (error == IMG_OK) CTi::SymetrieV(m_img);
			break;

		case BMP:
		case PGM:
		case PPM:
			Fl_Shared_Image *flImg = Fl_Shared_Image::get(name);
			if (flImg) {
				m_img = new CImage((unsigned char *)flImg->data()[0], flImg->w(), flImg->h(), flImg->d()*8);
				flImg->release();
			}
			break;
	}
	// ===========================

	if (m_img && error == IMG_OK)
	{
		m_imgAff = m_img->Copy();
		size(m_imgAff->width, m_imgAff->height);
		m_zoom = 1;
		m_curseur[0] = m_imgAff->width/2;
		m_curseur[1] = m_imgAff->height/2;
		return true;
	}
	else {
		size(0, 0);
		m_img = NULL;
		m_imgAff = NULL;
		m_imgUndo = NULL;
		fl_alert("Impossible de charger cette image !\nFormat invalide.");
		return false;
	}
}

// ============================
// Effectue un zoom sur l'image
// ============================
void CImageBoxFL::Zoom(int zoom)
{
	m_curseur[0] /= m_zoom;
	m_curseur[1] /= m_zoom;
	m_zoom = zoom/100.0f;
	if (m_imgAff) delete m_imgAff;
	if (m_img) {
		Fl_Image *img,*temp;
		img = new Fl_RGB_Image(m_img->data, m_img->width, m_img->height, m_img->bitCount/8);
		temp = img->copy((int)(img->w()*m_zoom), (int)(img->h()*m_zoom));
		delete img;
		img = temp;
		m_imgAff = new CImage((unsigned char *)img->data()[0], img->w(), img->h(), img->d()*8);
		delete img;
		size(m_imgAff->width, m_imgAff->height);
		m_curseur[0] *= m_zoom;
		m_curseur[1] *= m_zoom;
	}
}

// ==================================================
// L'image a change donc on change l'image a afficher
// ==================================================
void CImageBoxFL::ImageChange()
{
	if (m_imgAff) delete m_imgAff;
	if (m_img) {
		Fl_Image *img,*temp;
		img = new Fl_RGB_Image(m_img->data, m_img->width, m_img->height, m_img->bitCount/8);
		temp = img->copy((int)(img->w()*m_zoom), (int)(img->h()*m_zoom));
		delete img;
		img = temp;
		m_imgAff = new CImage((unsigned char *)img->data()[0], img->w(), img->h(), img->d()*8);
		delete img;
		size(m_imgAff->width, m_imgAff->height);
	}
}

// ===================================
// Ferme l'image et arrete l'affichage
// ===================================
void CImageBoxFL::Close()
{
	if (m_imgAff) delete m_imgAff;
	if (m_img) delete m_img;
	if (m_imgUndo) delete m_img;
	size(0, 0);
	m_imgAff = NULL;
	m_img = NULL;
	m_imgUndo = NULL;
	m_curseur[0] = 0;
	m_curseur[1] = 0;
}

// =============================================
// Sauvegarde l'image courante dans l'image undo
// =============================================
void CImageBoxFL::SaveUndo()
{
	if (m_img)
	{
		if (m_imgUndo) delete m_imgUndo;
		m_imgUndo = m_img->Copy();
	}
}

// ===========================================
// Recupere l'image undo pour l'image courante
// ===========================================
void CImageBoxFL::Undo()
{
	if (m_imgUndo)
	{
		if (m_img) delete m_img;
		m_img = m_imgUndo->Copy();
	}
}

// =======================
// En attente d'evenements
// =======================
int CImageBoxFL::handle(int event)
{
	switch(event) {
		case FL_PUSH:
			//fl_alert("%d %d", Fl::event_x()-x(), Fl::event_y()-y());
			m_curseur[0] = Fl::event_x()-x();
			m_curseur[1] = Fl::event_y()-y();
			redraw();
			return 1;
		default:
			return Fl_Widget::handle(event);
	}
}

// ===============================
// Retourne la position du curseur
// ===============================
void CImageBoxFL::GetCurseur(int *x, int *y)
{
	*x = (int)(m_curseur[0]/m_zoom);
	*y = (int)(m_curseur[1]/m_zoom);
}


