/*
 * 08/06/2003
 *
 * imageboxFL.h
 * Box (FLTK) permettant d'afficher une image
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _IMAGEBOXFL_H
#define _IMAGEBOXFL_H

#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/fl_ask.h>
#include <FL/Fl_Shared_Image.H>
#include <FL/fl_draw.h>

#include "image.h"
#include "ti.h"

#define LUM 0
#define RED 1
#define GREEN 2
#define BLUE 3

// ##################
// Classe CImageBoxFL
// ##################
class CImageBoxFL : public Fl_Box
{
public:

	// ============
	// Constructeur
	// ============
	CImageBoxFL(int X,int Y,int W,int H,const char* L=0);

	// ===========
	// Destructeur
	// ===========
	~CImageBoxFL();

	// ===============
	// Dessine l'image
	// ===============
	void draw();

	// =========================
	// Change l'image a afficher
	// =========================
	bool SetImage(const char *name);

	// ================
	// Retourne l'image
	// ================
	CImage * GetImage() { return m_img; }

	// ==================================================
	// L'image a change donc on change l'image a afficher
	// ==================================================
	void ImageChange();

	// ============================
	// Effectue un zoom sur l'image
	// ============================
	void Zoom(int zoom);

	// ===================================
	// Ferme l'image et arrete l'affichage
	// ===================================
	void Close();

	// =============================================
	// Sauvegarde l'image courante dans l'image undo
	// =============================================
	void SaveUndo();

	// ===========================================
	// Recupere l'image undo pour l'image courante
	// ===========================================
	void Undo();

	// =======================
	// En attente d'evenements
	// =======================
	int handle(int event);

	// ===============================
	// Retourne la position du curseur
	// ===============================
	void GetCurseur(int *x, int *y);

private:

	// Zoom sur l'image
	float m_zoom;
	// Image
	CImage *m_img;
	// Image affichee
	CImage *m_imgAff;
	// Image avant derniere action
	CImage *m_imgUndo;
	// position du curseur
	int m_curseur[2];
};

#endif




