/*
 * 08/07/2003
 *
 * ti.cpp
 * Classe permettant de traiter les images
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "ti.h"

// ==========================================
// Effectue le traitement demande sur l'image
// ==========================================
void CTi::Traitement(CImage *img, int traitement, RGB c, int ptx, int pty, float val)
{
	switch (traitement)
	{
	case NEGATIF: Negatif(img); break;
	case DISTANCE4POINT:  Distance4Point(img,c,ptx,pty,val); break;
	case DISTANCE8POINT:  Distance8Point(img,c,ptx,pty,val); break;
	case DENTSDESCIE: DentsDeScie(img); break;
	case SEUILLAGEAL: SeuillageAl(img); break;
	case SEUILLAGEFIXE: SeuillageFixe(img,val); break;
	case EGALHISTO: EgalHisto(img); break;
	case ETIREMENTHISTO: EtirementHisto(img); break;
	}
}

// ==============================================================
// Change la luminosit d'une image, ou l'intensite d'une couleur
// ==============================================================
void CTi::Luminosite(CImage *img, int lum, int r, int g, int b)
{	
	int tmp;
	int np = (img->bitCount/8);
	long d = img->width * img->height * np;
	
	if (np == 1 && lum != 0)
	for (long i = 0; i < d; i+=np)
	{
		tmp = (int)(img->data[i] * (1 + (float)(lum)/100));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i] = tmp;
	}
	else if (np == 3 && (lum!=0 || r!=0 || g!=0 || b!=0))
	for (long i = 0; i < d; i+=np)
	{
		tmp = (int)(img->data[i] * (1 + (float)(lum+r)/100));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i] = tmp;
		tmp = (int)(img->data[i+1] * (1 + (float)(lum+g)/100));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i+1] = tmp;
		tmp = (int)(img->data[i+2] * (1 + (float)(lum+b)/100));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i+2] = tmp;
	}

}

// ===============================
// Change le contraste d'une image
// ===============================
void CTi::Contraste(CImage *img, int val)
{
	int tmp;
	int np = (img->bitCount/8);
	long d = img->width * img->height * np;
	
	if (np == 1 && val != 0)
	for (long i = 0; i < d; i+=np)
	{
		tmp = (int)(img->data[i] + (float)(val)/100*(img->data[i]-127));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i] = tmp;
	}
	else if (np == 3 && val != 0)
	for (long i = 0; i < d; i+=np)
	{
		tmp = (int)(img->data[i] + (float)(val)/100*(img->data[i]-127));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i] = tmp;
		tmp = (int)(img->data[i+1] + (float)(val)/100*(img->data[i+1]-127));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i+1] = tmp;
		tmp = (int)(img->data[i+2] + (float)(val)/100*(img->data[i+2]-127));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i+2] = tmp;
	}
}

// =================================
// Change la saturation des couleurs
// =================================
void CTi::Saturation(CImage *img, int val)
{	
	int tmp;
	int np = (img->bitCount/8);
	long d = img->width * img->height * np;
	float m;
	if (np == 3 && val != 0)
	for (long i = 0; i < d; i+=np)
	{
		m = (float)(img->data[i]+img->data[i+1]+img->data[i+2])/3;
		tmp = (int)(img->data[i] + (float)(val)/100*(img->data[i]-m));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i] = tmp;
		tmp = (int)(img->data[i+1] + (float)(val)/100*(img->data[i+1]-m));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i+1] = tmp;
		tmp = (int)(img->data[i+2] + (float)(val)/100*(img->data[i+2]-m));
		if (tmp>255) tmp=255 ;
		else if (tmp<0) tmp=0 ;
		img->data[i+2] = tmp;
	}
}

// ==================
// Negatif de l'image
// ==================
void CTi::Negatif(CImage *img) 
{
	int np = (img->bitCount/8);
	long d = img->width * img->height * np;
	for (long i = 0; i < d; i++)
		img->data[i] = 255 - img->data[i];
}

// ==============================================================
// Transforme une image 24 bits en image 8 bits (niveaux de gris)
// ==============================================================
void CTi::NiveauxGris(CImage *img)
{
	if (img->bitCount == 24)
	{
		CImage *temp = new CImage(img->width, img->height, 8);
		int np = (img->bitCount/8);
		long d = img->width * img->height * np;
		long j = 0;
		for (long i = 0; i < d; i+=np) {
			temp->data[j] = (int)(img->data[i]*0.707+img->data[i+1]*0.202+img->data[i+2]*0.091);
			j++;
		}
		delete img;
		img = temp->Copy();
		delete temp;
	}
}

// =============================================================
// Transforme une image 8 bits (niveau de gris) en image 24 bits 
// =============================================================
void CTi::CouleursRVB(CImage *img)
{
	if (img->bitCount == 8)
	{
		CImage *temp = new CImage(img->width, img->height, 24);
		int np = (img->bitCount/8);
		long d = img->width * img->height * np;
		long j = 0;
		for (long i = 0; i < d; i+=np) {
			temp->data[j] = img->data[i];j++;
			temp->data[j] = img->data[i];j++;
			temp->data[j] = img->data[i];j++;
		}
		delete img;
		img = temp->Copy();
		delete temp;
	}
}

// =======================
// Rotation de 90 horaire
// =======================
void CTi::Rot90h(CImage *img)
{
	CImage *temp = new CImage(img->height, img->width, img->bitCount);
	int np = (img->bitCount/8);
	long d = img->width * img->height * np;
	int x,y;
	int width = img->width*np;
	for (long i = 0; i < d; i+=np) {
		x = i % width;
		y = i / width;
		temp->data[x*img->height + (img->height-1-y)*np] = img->data[i];
		if (np == 3) {
			temp->data[x*img->height + (img->height-1-y)*np + 1] = img->data[i + 1];
			temp->data[x*img->height + (img->height-1-y)*np + 2] = img->data[i + 2];
		}
	}

	delete img;
	img = temp->Copy();
	delete temp;
}

// ===========================
// Rotation de 90 antihoraire
// ===========================
void CTi::Rot90a(CImage *img)
{
	Rot90h(img);Rot180(img);
}

// ================
// Rotation de 180
// ================
void CTi::Rot180(CImage *img)
{
	Rot90h(img);Rot90h(img);
}


// =======================
// Symetrie d'axe vertical
// =======================
void CTi::SymetrieV(CImage *img)
{
	CImage *tmp = new CImage(img->width, img->height, img->bitCount);
	int np = (img->bitCount/8);
	int h = img->height;
	int w = img->width;
	for (int i = 0; i < h; i++)
		memcpy(tmp->data + np*i*w, img->data + (h-1)*w*np - i*np*w, w*np*sizeof(unsigned char));

	memcpy((void*)img->data, (void*)tmp->data, w*h*np*sizeof(unsigned char)); 
		
	delete tmp;
}

// =========================
// Symetrie d'axe horizontal
// =========================
void CTi::SymetrieH(CImage *img)
{
	SymetrieV(img);Rot180(img);	
}

// =====================
// Redimensionne l'image
// =====================
void CTi::Redimensionne(CImage *img, int w, int h)
{
	if (w!=img->width || h!=img->height)
	{
		Fl_Image *temp2,*temp;
		temp2 = new Fl_RGB_Image(img->data, img->width, img->height, img->bitCount/8);
		temp = temp2->copy(w, h);
		delete temp2;
		delete img;
		img = new CImage((unsigned char *)temp->data()[0], temp->w(), temp->h(), temp->d()*8);
		delete temp;
	}
}

// =====================
// 4 distance fond/point
// =====================
void CTi::Distance4Point(CImage *img, RGB c, int ptx, int pty, float val) 
{
	int x, y, d;
	int np = (img->bitCount/8);
	long size = img->width * img->height * np;
	float r;

	ptx *= np;
	for (long i = 0; i < size; i+=np) {
		y = i/(img->width*np);
		x = i%(img->width*np);
		// Calcul de la distance
		d = (int)((abs(x - ptx)/np)/val + abs(y - pty)/val);
		if (d > 255) d = 255;
		
		if (np == 1) {
			img->data[i] = d;
		}
		else {
			if (c.r) img->data[i] = d;
			if (c.g) img->data[i+1] = d;
			if (c.b) img->data[i+2] = d;
		}
	}
}

// =====================
// 8 distance fond/point
// =====================
void CTi::Distance8Point(CImage *img, RGB c, int ptx, int pty, float val) 
{
	int x, y, d;
	int np = (img->bitCount/8);
	long size = img->width * img->height * np;
	float r;

	ptx *= np;
	for (long i = 0; i < size; i+=np) {
		y = i/(img->width*np);
		x = i%(img->width*np);
		// Calcul de la distance
		d = (int)(MAX((abs(x - ptx)/np)/val, abs(y - pty)/val));
		if (d > 255) d = 255;
		
		if (np == 1) {
			img->data[i] = d;
		}
		else {
			if (c.r) img->data[i] = d;
			if (c.g) img->data[i+1] = d;
			if (c.b) img->data[i+2] = d;
		}
	}
}

// =============
// Dents de scie
// =============
void CTi::DentsDeScie(CImage *img)
{
	int tmp;
	int np = (img->bitCount/8);
	long size = img->width * img->height * np;

	if(np==3)
	for(long i=0; i<size; i+=3)
	{
		tmp = (int)(img->data[i]*0.707+img->data[i+1]*0.202+img->data[i+2]*0.091);
		img->data[i] = (tmp*8)%256;
		img->data[i+1] = (tmp*8)%256;
		img->data[i+2] = (tmp*8)%256;
	}
	else
	for(long i=0;i<size;i++)
		img->data[i] = (img->data[i]*8)%256;
}

// ===================
// Seuillage aleatoire
// ===================
void CTi::SeuillageAl(CImage *img) 
{
	int np = (img->bitCount/8);
	long size = img->width * img->height * np;
	int s;
	unsigned char ng;
	srand((unsigned)time(NULL));
	for(long i=0; i<size; i+=np)
	{
		if (np==3) ng = (int)(img->data[i]*0.707+img->data[i+1]*0.202+img->data[i+2]*0.091);
		else ng = img->data[i];
		s = rand()%256;
		if (ng > s ) ng = 255; 
		else ng = 0;
		for(int j=0;j<np;j++) img->data[i+j] = ng;
	}
}

// ==============
// Seuillage fixe
// ==============
void CTi::SeuillageFixe(CImage *img, float val) 
{
	int np = (img->bitCount/8);
	long size = img->width * img->height * np;
	int s = (int)(255.0/10.0 * val);
	unsigned char ng;
	for(long i=0; i<size; i+=np)
	{
		if (np==3) ng = (int)(img->data[i]*0.707+img->data[i+1]*0.202+img->data[i+2]*0.091);
		else ng = img->data[i];
		if (ng > s ) ng = 255; 
		else ng = 0;
		for(int j=0;j<np;j++) img->data[i+j] = ng;
	}
}

// =========================
// Egalisation d'histogramme
// =========================
void CTi::EgalHisto(CImage *img) 
{
	long i;
	int D,tmp,ng;
	int np = (img->bitCount/8);
	long size = img->width * img->height * np;
	int histo[256];
	int trans[256];
	
	for (i=0; i<256; i++) {
		histo[i] = 0;
	}
	for (i=0; i<size; i+=np) {
		ng = (int)(img->data[i]*0.707+img->data[i+1]*0.202+img->data[i+2]*0.091);
		histo[ng]++;
	}
	D = (img->width*img->height)/255;
	tmp = histo[0];
	trans[0] = 0;
	for (i=1; i<256; i++) {
		tmp += histo[i];
		trans[i] = tmp/D;	
	}
	for (i=0; i<size; i++) {
		img->data[i] = trans[img->data[i]];
	}
}

// =======================
// Etirement d'histogramme
// =======================
void CTi::EtirementHisto(CImage *img) 
{
	int np = (img->bitCount/8);
	long size = img->width * img->height * np;

	int* bitmap = new int[img->width * img->height];
	int tmp;
	long i;
	int z=0, max = 0, min = 255;
	float transfert[256];

	for(i=0; i<size; i+=np)
	{	
		if (np == 3)
			bitmap[z] = (int)(img->data[i]*0.707 + img->data[i+1]*0.202 + img->data[i+2]*0.091);
		else bitmap[z] = img->data[i];
		if(bitmap[z]>max) max = bitmap[z];
		if(bitmap[z]<min) min = bitmap[z];
		z++;
	}

	for(i=0;i<256;i++)
	{
		tmp = (max-min) == 0 ? 1 : max-min ;
		transfert[i] = (float)(((i-min)*255)/(tmp));
	}

	z=0;

	for(i=0; i<size; i+=np)
	{	
		tmp = (int)((float)img->data[i]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) img->data[i] = 255;
		else if(tmp<0) img->data[i] = 0;
		else img->data[i] = tmp;
		
		if (np == 3) {
		tmp = (int)((float)img->data[i+1]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) img->data[i+1] = 255;
		else if(tmp<0) img->data[i+1] = 0;
		else img->data[i+1] = tmp;


		tmp = (int)((float)img->data[i+2]*transfert[bitmap[z]]/bitmap[z]);
		if(tmp>255) img->data[i+2] = 255;
		else if(tmp<0) img->data[i+2] = 0;
		else img->data[i+2] = tmp;
		}
		z++;
	}	
	delete bitmap;
}

