/*
 * 08/07/2003
 *
 * ti.h
 * Classe permettant de traiter les images
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _TI_H
#define _TI_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include <FL/Fl.H>
#include <FL/fl_ask.h>
#include <FL/fl_draw.h>
#include <FL/Fl_Shared_Image.H>


#include "image.h"

#define NEGATIF 0x01
#define DISTANCE4POINT 0x02
#define DISTANCE8POINT 0x03
#define DENTSDESCIE 0x04
#define SEUILLAGEAL 0x05
#define SEUILLAGEFIXE 0x06
#define EGALHISTO 0x07
#define ETIREMENTHISTO 0x08

typedef struct sRGB {
	int r;
	int g;
	int b;
} RGB;

#define MAX(X,Y) (((X) < (Y)) ? (Y) : (X))
#define MIN(X,Y) (((X) < (Y)) ? (X) : (Y))

// ##########
// Classe CTi
// ##########
class CTi
{
public:

	// ==========================================
	// Effectue le traitement demande sur l'image
	// ==========================================
	void static Traitement(CImage *img, int traitement, RGB c, int ptx, int pty, float val);

	// ==============================================================
	// Change la luminosit d'une image, ou l'intensite d'une couleur
	// ==============================================================
	void static Luminosite(CImage *img, int lum, int r, int g, int b);

	// ===============================
	// Change le contraste d'une image
	// ===============================
	void static Contraste(CImage *img, int val);

	// =================================
	// Change la saturation des couleurs
	// =================================
	void static Saturation(CImage *img, int val);

	// ==================
	// Negatif de l'image
	// ==================
	void static Negatif(CImage *img);

	// ==============================================================
	// Transforme une image 24 bits en image 8 bits (niveaux de gris)
	// ==============================================================
	void static NiveauxGris(CImage *img);

	// =============================================================
	// Transforme une image 8 bits (niveau de gris) en image 24 bits 
	// =============================================================
	void static CouleursRVB(CImage *img);

	// =======================
	// Rotation de 90 horaire
	// =======================
	void static Rot90h(CImage *img);

	// ===========================
	// Rotation de 90 antihoraire
	// ===========================
	void static Rot90a(CImage *img);

	// ================
	// Rotation de 180
	// ================
	void static Rot180(CImage *img);

	// =======================
	// Symetrie d'axe vertical
	// =======================
	void static SymetrieV(CImage *img);

	// =========================
	// Symetrie d'axe horizontal
	// =========================
	void static SymetrieH(CImage *img);

	// =====================
	// Redimensionne l'image
	// =====================
	void static Redimensionne(CImage *img, int w, int h);

	// =====================
	// 4 distance fond/point
	// =====================
	void static Distance4Point(CImage *img, RGB c, int ptx, int pty, float val);

	// =====================
	// 8 distance fond/point
	// =====================
	void static Distance8Point(CImage *img, RGB c, int ptx, int pty, float val);

	// =============
	// Dents de scie
	// =============
	void static DentsDeScie(CImage *img);

	// ===================
	// Seuillage aleatoire
	// ===================
	void static SeuillageAl(CImage *img);

	// ==============
	// Seuillage fixe
	// ==============
	void static SeuillageFixe(CImage *img, float val);

	// =========================
	// Egalisation d'histogramme
	// =========================
	void static EgalHisto(CImage *img);

	// =======================
	// Etirement d'histogramme
	// =======================
	void static EtirementHisto(CImage *img);

private:

};

#endif
